# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetServiceEnvironmentsResult',
    'AwaitableGetServiceEnvironmentsResult',
    'get_service_environments',
    'get_service_environments_output',
]

@pulumi.output_type
class GetServiceEnvironmentsResult:
    """
    A collection of values returned by getServiceEnvironments.
    """
    def __init__(__self__, compartment_id=None, display_name=None, filters=None, id=None, service_environment_collections=None, service_environment_id=None, service_environment_type=None):
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if service_environment_collections and not isinstance(service_environment_collections, list):
            raise TypeError("Expected argument 'service_environment_collections' to be a list")
        pulumi.set(__self__, "service_environment_collections", service_environment_collections)
        if service_environment_id and not isinstance(service_environment_id, str):
            raise TypeError("Expected argument 'service_environment_id' to be a str")
        pulumi.set(__self__, "service_environment_id", service_environment_id)
        if service_environment_type and not isinstance(service_environment_type, str):
            raise TypeError("Expected argument 'service_environment_type' to be a str")
        pulumi.set(__self__, "service_environment_type", service_environment_type)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) for the compartment.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[str]:
        """
        Display name of the service. For example, "Oracle Retail Order Management Cloud Service".
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetServiceEnvironmentsFilterResult']]:
        return pulumi.get(self, "filters")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="serviceEnvironmentCollections")
    def service_environment_collections(self) -> Sequence['outputs.GetServiceEnvironmentsServiceEnvironmentCollectionResult']:
        """
        The list of service_environment_collection.
        """
        return pulumi.get(self, "service_environment_collections")

    @property
    @pulumi.getter(name="serviceEnvironmentId")
    def service_environment_id(self) -> Optional[str]:
        return pulumi.get(self, "service_environment_id")

    @property
    @pulumi.getter(name="serviceEnvironmentType")
    def service_environment_type(self) -> Optional[str]:
        return pulumi.get(self, "service_environment_type")


class AwaitableGetServiceEnvironmentsResult(GetServiceEnvironmentsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetServiceEnvironmentsResult(
            compartment_id=self.compartment_id,
            display_name=self.display_name,
            filters=self.filters,
            id=self.id,
            service_environment_collections=self.service_environment_collections,
            service_environment_id=self.service_environment_id,
            service_environment_type=self.service_environment_type)


def get_service_environments(compartment_id: Optional[str] = None,
                             display_name: Optional[str] = None,
                             filters: Optional[Sequence[pulumi.InputType['GetServiceEnvironmentsFilterArgs']]] = None,
                             service_environment_id: Optional[str] = None,
                             service_environment_type: Optional[str] = None,
                             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetServiceEnvironmentsResult:
    """
    This data source provides the list of Service Environments in Oracle Cloud Infrastructure Service Manager Proxy service.

    List the details of Software as a Service (SaaS) environments provisioned by Service Manager.
    Information includes the service instance endpoints and service definition details.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_service_environments = oci.ServiceManagerProxy.get_service_environments(compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        service_environment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        service_environment_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) for the compartment.
    :param str display_name: The display name of the resource.
    :param str service_environment_id: The unique identifier associated with the service environment. 
           
           **Note:** Not an [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm).
    :param str service_environment_type: The environment's service definition type.  For example, "RGBUOROMS" is the service definition type for "Oracle Retail Order Management Cloud Service".
    """
    __args__ = dict()
    __args__['compartmentId'] = compartment_id
    __args__['displayName'] = display_name
    __args__['filters'] = filters
    __args__['serviceEnvironmentId'] = service_environment_id
    __args__['serviceEnvironmentType'] = service_environment_type
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:ServiceManagerProxy/getServiceEnvironments:getServiceEnvironments', __args__, opts=opts, typ=GetServiceEnvironmentsResult).value

    return AwaitableGetServiceEnvironmentsResult(
        compartment_id=__ret__.compartment_id,
        display_name=__ret__.display_name,
        filters=__ret__.filters,
        id=__ret__.id,
        service_environment_collections=__ret__.service_environment_collections,
        service_environment_id=__ret__.service_environment_id,
        service_environment_type=__ret__.service_environment_type)


@_utilities.lift_output_func(get_service_environments)
def get_service_environments_output(compartment_id: Optional[pulumi.Input[str]] = None,
                                    display_name: Optional[pulumi.Input[Optional[str]]] = None,
                                    filters: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetServiceEnvironmentsFilterArgs']]]]] = None,
                                    service_environment_id: Optional[pulumi.Input[Optional[str]]] = None,
                                    service_environment_type: Optional[pulumi.Input[Optional[str]]] = None,
                                    opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetServiceEnvironmentsResult]:
    """
    This data source provides the list of Service Environments in Oracle Cloud Infrastructure Service Manager Proxy service.

    List the details of Software as a Service (SaaS) environments provisioned by Service Manager.
    Information includes the service instance endpoints and service definition details.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_service_environments = oci.ServiceManagerProxy.get_service_environments(compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        service_environment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        service_environment_type=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) for the compartment.
    :param str display_name: The display name of the resource.
    :param str service_environment_id: The unique identifier associated with the service environment. 
           
           **Note:** Not an [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm).
    :param str service_environment_type: The environment's service definition type.  For example, "RGBUOROMS" is the service definition type for "Oracle Retail Order Management Cloud Service".
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetDeploymentResult',
    'AwaitableGetDeploymentResult',
    'get_deployment',
    'get_deployment_output',
]

@pulumi.output_type
class GetDeploymentResult:
    """
    A collection of values returned by getDeployment.
    """
    def __init__(__self__, compartment_id=None, cpu_core_count=None, defined_tags=None, deployment_backup_id=None, deployment_diagnostic_datas=None, deployment_id=None, deployment_type=None, deployment_url=None, description=None, display_name=None, fqdn=None, freeform_tags=None, id=None, is_auto_scaling_enabled=None, is_healthy=None, is_latest_version=None, is_public=None, is_storage_utilization_limit_exceeded=None, license_model=None, lifecycle_details=None, lifecycle_sub_state=None, maintenance_windows=None, next_maintenance_action_type=None, next_maintenance_description=None, nsg_ids=None, ogg_datas=None, private_ip_address=None, public_ip_address=None, state=None, storage_utilization_in_bytes=None, subnet_id=None, system_tags=None, time_created=None, time_of_next_maintenance=None, time_updated=None, time_upgrade_required=None):
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if cpu_core_count and not isinstance(cpu_core_count, int):
            raise TypeError("Expected argument 'cpu_core_count' to be a int")
        pulumi.set(__self__, "cpu_core_count", cpu_core_count)
        if defined_tags and not isinstance(defined_tags, dict):
            raise TypeError("Expected argument 'defined_tags' to be a dict")
        pulumi.set(__self__, "defined_tags", defined_tags)
        if deployment_backup_id and not isinstance(deployment_backup_id, str):
            raise TypeError("Expected argument 'deployment_backup_id' to be a str")
        pulumi.set(__self__, "deployment_backup_id", deployment_backup_id)
        if deployment_diagnostic_datas and not isinstance(deployment_diagnostic_datas, list):
            raise TypeError("Expected argument 'deployment_diagnostic_datas' to be a list")
        pulumi.set(__self__, "deployment_diagnostic_datas", deployment_diagnostic_datas)
        if deployment_id and not isinstance(deployment_id, str):
            raise TypeError("Expected argument 'deployment_id' to be a str")
        pulumi.set(__self__, "deployment_id", deployment_id)
        if deployment_type and not isinstance(deployment_type, str):
            raise TypeError("Expected argument 'deployment_type' to be a str")
        pulumi.set(__self__, "deployment_type", deployment_type)
        if deployment_url and not isinstance(deployment_url, str):
            raise TypeError("Expected argument 'deployment_url' to be a str")
        pulumi.set(__self__, "deployment_url", deployment_url)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if fqdn and not isinstance(fqdn, str):
            raise TypeError("Expected argument 'fqdn' to be a str")
        pulumi.set(__self__, "fqdn", fqdn)
        if freeform_tags and not isinstance(freeform_tags, dict):
            raise TypeError("Expected argument 'freeform_tags' to be a dict")
        pulumi.set(__self__, "freeform_tags", freeform_tags)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if is_auto_scaling_enabled and not isinstance(is_auto_scaling_enabled, bool):
            raise TypeError("Expected argument 'is_auto_scaling_enabled' to be a bool")
        pulumi.set(__self__, "is_auto_scaling_enabled", is_auto_scaling_enabled)
        if is_healthy and not isinstance(is_healthy, bool):
            raise TypeError("Expected argument 'is_healthy' to be a bool")
        pulumi.set(__self__, "is_healthy", is_healthy)
        if is_latest_version and not isinstance(is_latest_version, bool):
            raise TypeError("Expected argument 'is_latest_version' to be a bool")
        pulumi.set(__self__, "is_latest_version", is_latest_version)
        if is_public and not isinstance(is_public, bool):
            raise TypeError("Expected argument 'is_public' to be a bool")
        pulumi.set(__self__, "is_public", is_public)
        if is_storage_utilization_limit_exceeded and not isinstance(is_storage_utilization_limit_exceeded, bool):
            raise TypeError("Expected argument 'is_storage_utilization_limit_exceeded' to be a bool")
        pulumi.set(__self__, "is_storage_utilization_limit_exceeded", is_storage_utilization_limit_exceeded)
        if license_model and not isinstance(license_model, str):
            raise TypeError("Expected argument 'license_model' to be a str")
        pulumi.set(__self__, "license_model", license_model)
        if lifecycle_details and not isinstance(lifecycle_details, str):
            raise TypeError("Expected argument 'lifecycle_details' to be a str")
        pulumi.set(__self__, "lifecycle_details", lifecycle_details)
        if lifecycle_sub_state and not isinstance(lifecycle_sub_state, str):
            raise TypeError("Expected argument 'lifecycle_sub_state' to be a str")
        pulumi.set(__self__, "lifecycle_sub_state", lifecycle_sub_state)
        if maintenance_windows and not isinstance(maintenance_windows, list):
            raise TypeError("Expected argument 'maintenance_windows' to be a list")
        pulumi.set(__self__, "maintenance_windows", maintenance_windows)
        if next_maintenance_action_type and not isinstance(next_maintenance_action_type, str):
            raise TypeError("Expected argument 'next_maintenance_action_type' to be a str")
        pulumi.set(__self__, "next_maintenance_action_type", next_maintenance_action_type)
        if next_maintenance_description and not isinstance(next_maintenance_description, str):
            raise TypeError("Expected argument 'next_maintenance_description' to be a str")
        pulumi.set(__self__, "next_maintenance_description", next_maintenance_description)
        if nsg_ids and not isinstance(nsg_ids, list):
            raise TypeError("Expected argument 'nsg_ids' to be a list")
        pulumi.set(__self__, "nsg_ids", nsg_ids)
        if ogg_datas and not isinstance(ogg_datas, list):
            raise TypeError("Expected argument 'ogg_datas' to be a list")
        pulumi.set(__self__, "ogg_datas", ogg_datas)
        if private_ip_address and not isinstance(private_ip_address, str):
            raise TypeError("Expected argument 'private_ip_address' to be a str")
        pulumi.set(__self__, "private_ip_address", private_ip_address)
        if public_ip_address and not isinstance(public_ip_address, str):
            raise TypeError("Expected argument 'public_ip_address' to be a str")
        pulumi.set(__self__, "public_ip_address", public_ip_address)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)
        if storage_utilization_in_bytes and not isinstance(storage_utilization_in_bytes, str):
            raise TypeError("Expected argument 'storage_utilization_in_bytes' to be a str")
        pulumi.set(__self__, "storage_utilization_in_bytes", storage_utilization_in_bytes)
        if subnet_id and not isinstance(subnet_id, str):
            raise TypeError("Expected argument 'subnet_id' to be a str")
        pulumi.set(__self__, "subnet_id", subnet_id)
        if system_tags and not isinstance(system_tags, dict):
            raise TypeError("Expected argument 'system_tags' to be a dict")
        pulumi.set(__self__, "system_tags", system_tags)
        if time_created and not isinstance(time_created, str):
            raise TypeError("Expected argument 'time_created' to be a str")
        pulumi.set(__self__, "time_created", time_created)
        if time_of_next_maintenance and not isinstance(time_of_next_maintenance, str):
            raise TypeError("Expected argument 'time_of_next_maintenance' to be a str")
        pulumi.set(__self__, "time_of_next_maintenance", time_of_next_maintenance)
        if time_updated and not isinstance(time_updated, str):
            raise TypeError("Expected argument 'time_updated' to be a str")
        pulumi.set(__self__, "time_updated", time_updated)
        if time_upgrade_required and not isinstance(time_upgrade_required, str):
            raise TypeError("Expected argument 'time_upgrade_required' to be a str")
        pulumi.set(__self__, "time_upgrade_required", time_upgrade_required)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment being referenced.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="cpuCoreCount")
    def cpu_core_count(self) -> int:
        """
        The Minimum number of OCPUs to be made available for this Deployment.
        """
        return pulumi.get(self, "cpu_core_count")

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Mapping[str, Any]:
        """
        Tags defined for this resource. Each key is predefined and scoped to a namespace.  Example: `{"foo-namespace.bar-key": "value"}`
        """
        return pulumi.get(self, "defined_tags")

    @property
    @pulumi.getter(name="deploymentBackupId")
    def deployment_backup_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the backup being referenced.
        """
        return pulumi.get(self, "deployment_backup_id")

    @property
    @pulumi.getter(name="deploymentDiagnosticDatas")
    def deployment_diagnostic_datas(self) -> Sequence['outputs.GetDeploymentDeploymentDiagnosticDataResult']:
        """
        Information regarding the deployment diagnostic collection
        """
        return pulumi.get(self, "deployment_diagnostic_datas")

    @property
    @pulumi.getter(name="deploymentId")
    def deployment_id(self) -> str:
        return pulumi.get(self, "deployment_id")

    @property
    @pulumi.getter(name="deploymentType")
    def deployment_type(self) -> str:
        """
        The type of deployment, which can be any one of the Allowed values.  NOTE: Use of the value 'OGG' is maintained for backward compatibility purposes.  Its use is discouraged in favor of 'DATABASE_ORACLE'.
        """
        return pulumi.get(self, "deployment_type")

    @property
    @pulumi.getter(name="deploymentUrl")
    def deployment_url(self) -> str:
        """
        The URL of a resource.
        """
        return pulumi.get(self, "deployment_url")

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        Metadata about this specific object.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        """
        An object's Display Name.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def fqdn(self) -> str:
        """
        A three-label Fully Qualified Domain Name (FQDN) for a resource.
        """
        return pulumi.get(self, "fqdn")

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Mapping[str, Any]:
        """
        A simple key-value pair that is applied without any predefined name, type, or scope. Exists for cross-compatibility only.  Example: `{"bar-key": "value"}`
        """
        return pulumi.get(self, "freeform_tags")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the deployment being referenced.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="isAutoScalingEnabled")
    def is_auto_scaling_enabled(self) -> bool:
        """
        Indicates if auto scaling is enabled for the Deployment's CPU core count.
        """
        return pulumi.get(self, "is_auto_scaling_enabled")

    @property
    @pulumi.getter(name="isHealthy")
    def is_healthy(self) -> bool:
        """
        True if all of the aggregate resources are working correctly.
        """
        return pulumi.get(self, "is_healthy")

    @property
    @pulumi.getter(name="isLatestVersion")
    def is_latest_version(self) -> bool:
        """
        Indicates if the resource is the the latest available version.
        """
        return pulumi.get(self, "is_latest_version")

    @property
    @pulumi.getter(name="isPublic")
    def is_public(self) -> bool:
        """
        True if this object is publicly available.
        """
        return pulumi.get(self, "is_public")

    @property
    @pulumi.getter(name="isStorageUtilizationLimitExceeded")
    def is_storage_utilization_limit_exceeded(self) -> bool:
        """
        Indicator will be true if the amount of storage being utilized exceeds the allowable storage utilization limit.  Exceeding the limit may be an indication of a misconfiguration of the deployment's GoldenGate service.
        """
        return pulumi.get(self, "is_storage_utilization_limit_exceeded")

    @property
    @pulumi.getter(name="licenseModel")
    def license_model(self) -> str:
        """
        The Oracle license model that applies to a Deployment.
        """
        return pulumi.get(self, "license_model")

    @property
    @pulumi.getter(name="lifecycleDetails")
    def lifecycle_details(self) -> str:
        """
        Describes the object's current state in detail. For example, it can be used to provide actionable information for a resource in a Failed state.
        """
        return pulumi.get(self, "lifecycle_details")

    @property
    @pulumi.getter(name="lifecycleSubState")
    def lifecycle_sub_state(self) -> str:
        """
        Possible GGS lifecycle sub-states.
        """
        return pulumi.get(self, "lifecycle_sub_state")

    @property
    @pulumi.getter(name="maintenanceWindows")
    def maintenance_windows(self) -> Sequence['outputs.GetDeploymentMaintenanceWindowResult']:
        """
        Defines the maintenance window, when automatic actions can be performed.
        """
        return pulumi.get(self, "maintenance_windows")

    @property
    @pulumi.getter(name="nextMaintenanceActionType")
    def next_maintenance_action_type(self) -> str:
        """
        Type of the next maintenance.
        """
        return pulumi.get(self, "next_maintenance_action_type")

    @property
    @pulumi.getter(name="nextMaintenanceDescription")
    def next_maintenance_description(self) -> str:
        """
        Description of the next maintenance.
        """
        return pulumi.get(self, "next_maintenance_description")

    @property
    @pulumi.getter(name="nsgIds")
    def nsg_ids(self) -> Sequence[str]:
        """
        An array of Network Security Group OCIDs used to define network access for either Deployments or Connections.
        """
        return pulumi.get(self, "nsg_ids")

    @property
    @pulumi.getter(name="oggDatas")
    def ogg_datas(self) -> Sequence['outputs.GetDeploymentOggDataResult']:
        """
        Deployment Data for an OggDeployment
        """
        return pulumi.get(self, "ogg_datas")

    @property
    @pulumi.getter(name="privateIpAddress")
    def private_ip_address(self) -> str:
        """
        The private IP address in the customer's VCN representing the access point for the associated endpoint service in the GoldenGate service VCN.
        """
        return pulumi.get(self, "private_ip_address")

    @property
    @pulumi.getter(name="publicIpAddress")
    def public_ip_address(self) -> str:
        """
        The public IP address representing the access point for the Deployment.
        """
        return pulumi.get(self, "public_ip_address")

    @property
    @pulumi.getter
    def state(self) -> str:
        """
        Possible lifecycle states.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="storageUtilizationInBytes")
    def storage_utilization_in_bytes(self) -> str:
        """
        The amount of storage being utilized (in bytes)
        """
        return pulumi.get(self, "storage_utilization_in_bytes")

    @property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the subnet being referenced.
        """
        return pulumi.get(self, "subnet_id")

    @property
    @pulumi.getter(name="systemTags")
    def system_tags(self) -> Mapping[str, Any]:
        """
        The system tags associated with this resource, if any. The system tags are set by Oracle Cloud Infrastructure services. Each key is predefined and scoped to namespaces.  For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{orcl-cloud: {free-tier-retain: true}}`
        """
        return pulumi.get(self, "system_tags")

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> str:
        """
        The time the resource was created. The format is defined by [RFC3339](https://tools.ietf.org/html/rfc3339), such as `2016-08-25T21:10:29.600Z`.
        """
        return pulumi.get(self, "time_created")

    @property
    @pulumi.getter(name="timeOfNextMaintenance")
    def time_of_next_maintenance(self) -> str:
        """
        The time of next maintenance schedule. The format is defined by [RFC3339](https://tools.ietf.org/html/rfc3339), such as `2016-08-25T21:10:29.600Z`.
        """
        return pulumi.get(self, "time_of_next_maintenance")

    @property
    @pulumi.getter(name="timeUpdated")
    def time_updated(self) -> str:
        """
        The time the resource was last updated. The format is defined by [RFC3339](https://tools.ietf.org/html/rfc3339), such as `2016-08-25T21:10:29.600Z`.
        """
        return pulumi.get(self, "time_updated")

    @property
    @pulumi.getter(name="timeUpgradeRequired")
    def time_upgrade_required(self) -> str:
        """
        Note: Deprecated: Use timeOfNextMaintenance instead, or related upgrade records  to check, when deployment will be forced to upgrade to a newer version. Old description: The date the existing version in use will no longer be considered as usable and an upgrade will be required.  This date is typically 6 months after the version was released for use by GGS.  The format is defined by [RFC3339](https://tools.ietf.org/html/rfc3339), such as `2016-08-25T21:10:29.600Z`.
        """
        return pulumi.get(self, "time_upgrade_required")


class AwaitableGetDeploymentResult(GetDeploymentResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDeploymentResult(
            compartment_id=self.compartment_id,
            cpu_core_count=self.cpu_core_count,
            defined_tags=self.defined_tags,
            deployment_backup_id=self.deployment_backup_id,
            deployment_diagnostic_datas=self.deployment_diagnostic_datas,
            deployment_id=self.deployment_id,
            deployment_type=self.deployment_type,
            deployment_url=self.deployment_url,
            description=self.description,
            display_name=self.display_name,
            fqdn=self.fqdn,
            freeform_tags=self.freeform_tags,
            id=self.id,
            is_auto_scaling_enabled=self.is_auto_scaling_enabled,
            is_healthy=self.is_healthy,
            is_latest_version=self.is_latest_version,
            is_public=self.is_public,
            is_storage_utilization_limit_exceeded=self.is_storage_utilization_limit_exceeded,
            license_model=self.license_model,
            lifecycle_details=self.lifecycle_details,
            lifecycle_sub_state=self.lifecycle_sub_state,
            maintenance_windows=self.maintenance_windows,
            next_maintenance_action_type=self.next_maintenance_action_type,
            next_maintenance_description=self.next_maintenance_description,
            nsg_ids=self.nsg_ids,
            ogg_datas=self.ogg_datas,
            private_ip_address=self.private_ip_address,
            public_ip_address=self.public_ip_address,
            state=self.state,
            storage_utilization_in_bytes=self.storage_utilization_in_bytes,
            subnet_id=self.subnet_id,
            system_tags=self.system_tags,
            time_created=self.time_created,
            time_of_next_maintenance=self.time_of_next_maintenance,
            time_updated=self.time_updated,
            time_upgrade_required=self.time_upgrade_required)


def get_deployment(deployment_id: Optional[str] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetDeploymentResult:
    """
    This data source provides details about a specific Deployment resource in Oracle Cloud Infrastructure Golden Gate service.

    Retrieves a deployment.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_deployment = oci.GoldenGate.get_deployment(deployment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str deployment_id: A unique Deployment identifier.
    """
    __args__ = dict()
    __args__['deploymentId'] = deployment_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:GoldenGate/getDeployment:getDeployment', __args__, opts=opts, typ=GetDeploymentResult).value

    return AwaitableGetDeploymentResult(
        compartment_id=__ret__.compartment_id,
        cpu_core_count=__ret__.cpu_core_count,
        defined_tags=__ret__.defined_tags,
        deployment_backup_id=__ret__.deployment_backup_id,
        deployment_diagnostic_datas=__ret__.deployment_diagnostic_datas,
        deployment_id=__ret__.deployment_id,
        deployment_type=__ret__.deployment_type,
        deployment_url=__ret__.deployment_url,
        description=__ret__.description,
        display_name=__ret__.display_name,
        fqdn=__ret__.fqdn,
        freeform_tags=__ret__.freeform_tags,
        id=__ret__.id,
        is_auto_scaling_enabled=__ret__.is_auto_scaling_enabled,
        is_healthy=__ret__.is_healthy,
        is_latest_version=__ret__.is_latest_version,
        is_public=__ret__.is_public,
        is_storage_utilization_limit_exceeded=__ret__.is_storage_utilization_limit_exceeded,
        license_model=__ret__.license_model,
        lifecycle_details=__ret__.lifecycle_details,
        lifecycle_sub_state=__ret__.lifecycle_sub_state,
        maintenance_windows=__ret__.maintenance_windows,
        next_maintenance_action_type=__ret__.next_maintenance_action_type,
        next_maintenance_description=__ret__.next_maintenance_description,
        nsg_ids=__ret__.nsg_ids,
        ogg_datas=__ret__.ogg_datas,
        private_ip_address=__ret__.private_ip_address,
        public_ip_address=__ret__.public_ip_address,
        state=__ret__.state,
        storage_utilization_in_bytes=__ret__.storage_utilization_in_bytes,
        subnet_id=__ret__.subnet_id,
        system_tags=__ret__.system_tags,
        time_created=__ret__.time_created,
        time_of_next_maintenance=__ret__.time_of_next_maintenance,
        time_updated=__ret__.time_updated,
        time_upgrade_required=__ret__.time_upgrade_required)


@_utilities.lift_output_func(get_deployment)
def get_deployment_output(deployment_id: Optional[pulumi.Input[str]] = None,
                          opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetDeploymentResult]:
    """
    This data source provides details about a specific Deployment resource in Oracle Cloud Infrastructure Golden Gate service.

    Retrieves a deployment.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_deployment = oci.GoldenGate.get_deployment(deployment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str deployment_id: A unique Deployment identifier.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['FusionEnvironmentArgs', 'FusionEnvironment']

@pulumi.input_type
class FusionEnvironmentArgs:
    def __init__(__self__, *,
                 compartment_id: pulumi.Input[str],
                 create_fusion_environment_admin_user_details: pulumi.Input['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs'],
                 display_name: pulumi.Input[str],
                 fusion_environment_family_id: pulumi.Input[str],
                 fusion_environment_type: pulumi.Input[str],
                 additional_language_packs: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 dns_prefix: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 kms_key_id: Optional[pulumi.Input[str]] = None,
                 maintenance_policy: Optional[pulumi.Input['FusionEnvironmentMaintenancePolicyArgs']] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRuleArgs']]]] = None):
        """
        The set of arguments for constructing a FusionEnvironment resource.
        :param pulumi.Input[str] compartment_id: (Updatable) The unique identifier (OCID) of the compartment where the Fusion Environment is located.
        :param pulumi.Input['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs'] create_fusion_environment_admin_user_details: The credentials for the Fusion Applications service administrator.
        :param pulumi.Input[str] display_name: (Updatable) FusionEnvironment Identifier can be renamed.
        :param pulumi.Input[str] fusion_environment_family_id: The unique identifier (OCID) of the Fusion Environment Family that the Fusion Environment belongs to.
        :param pulumi.Input[str] fusion_environment_type: The type of environment. Valid values are Production, Test, or Development.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] additional_language_packs: (Updatable) Language packs.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        :param pulumi.Input[str] dns_prefix: DNS prefix.
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        :param pulumi.Input[str] kms_key_id: (Updatable) byok kms keyId
        :param pulumi.Input['FusionEnvironmentMaintenancePolicyArgs'] maintenance_policy: (Updatable) The policy that specifies the maintenance and upgrade preferences for an environment. For more information about the options, see [Understanding Environment Maintenance](https://docs.cloud.oracle.com/iaas/Content/fusion-applications/plan-environment-family.htm#about-env-maintenance).
        :param pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRuleArgs']]] rules: (Updatable) Rules.
        """
        pulumi.set(__self__, "compartment_id", compartment_id)
        pulumi.set(__self__, "create_fusion_environment_admin_user_details", create_fusion_environment_admin_user_details)
        pulumi.set(__self__, "display_name", display_name)
        pulumi.set(__self__, "fusion_environment_family_id", fusion_environment_family_id)
        pulumi.set(__self__, "fusion_environment_type", fusion_environment_type)
        if additional_language_packs is not None:
            pulumi.set(__self__, "additional_language_packs", additional_language_packs)
        if defined_tags is not None:
            pulumi.set(__self__, "defined_tags", defined_tags)
        if dns_prefix is not None:
            pulumi.set(__self__, "dns_prefix", dns_prefix)
        if freeform_tags is not None:
            pulumi.set(__self__, "freeform_tags", freeform_tags)
        if kms_key_id is not None:
            pulumi.set(__self__, "kms_key_id", kms_key_id)
        if maintenance_policy is not None:
            pulumi.set(__self__, "maintenance_policy", maintenance_policy)
        if rules is not None:
            pulumi.set(__self__, "rules", rules)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> pulumi.Input[str]:
        """
        (Updatable) The unique identifier (OCID) of the compartment where the Fusion Environment is located.
        """
        return pulumi.get(self, "compartment_id")

    @compartment_id.setter
    def compartment_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "compartment_id", value)

    @property
    @pulumi.getter(name="createFusionEnvironmentAdminUserDetails")
    def create_fusion_environment_admin_user_details(self) -> pulumi.Input['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']:
        """
        The credentials for the Fusion Applications service administrator.
        """
        return pulumi.get(self, "create_fusion_environment_admin_user_details")

    @create_fusion_environment_admin_user_details.setter
    def create_fusion_environment_admin_user_details(self, value: pulumi.Input['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']):
        pulumi.set(self, "create_fusion_environment_admin_user_details", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Input[str]:
        """
        (Updatable) FusionEnvironment Identifier can be renamed.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="fusionEnvironmentFamilyId")
    def fusion_environment_family_id(self) -> pulumi.Input[str]:
        """
        The unique identifier (OCID) of the Fusion Environment Family that the Fusion Environment belongs to.
        """
        return pulumi.get(self, "fusion_environment_family_id")

    @fusion_environment_family_id.setter
    def fusion_environment_family_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "fusion_environment_family_id", value)

    @property
    @pulumi.getter(name="fusionEnvironmentType")
    def fusion_environment_type(self) -> pulumi.Input[str]:
        """
        The type of environment. Valid values are Production, Test, or Development.
        """
        return pulumi.get(self, "fusion_environment_type")

    @fusion_environment_type.setter
    def fusion_environment_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "fusion_environment_type", value)

    @property
    @pulumi.getter(name="additionalLanguagePacks")
    def additional_language_packs(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        (Updatable) Language packs.
        """
        return pulumi.get(self, "additional_language_packs")

    @additional_language_packs.setter
    def additional_language_packs(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "additional_language_packs", value)

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        """
        return pulumi.get(self, "defined_tags")

    @defined_tags.setter
    def defined_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "defined_tags", value)

    @property
    @pulumi.getter(name="dnsPrefix")
    def dns_prefix(self) -> Optional[pulumi.Input[str]]:
        """
        DNS prefix.
        """
        return pulumi.get(self, "dns_prefix")

    @dns_prefix.setter
    def dns_prefix(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dns_prefix", value)

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        """
        return pulumi.get(self, "freeform_tags")

    @freeform_tags.setter
    def freeform_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "freeform_tags", value)

    @property
    @pulumi.getter(name="kmsKeyId")
    def kms_key_id(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) byok kms keyId
        """
        return pulumi.get(self, "kms_key_id")

    @kms_key_id.setter
    def kms_key_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kms_key_id", value)

    @property
    @pulumi.getter(name="maintenancePolicy")
    def maintenance_policy(self) -> Optional[pulumi.Input['FusionEnvironmentMaintenancePolicyArgs']]:
        """
        (Updatable) The policy that specifies the maintenance and upgrade preferences for an environment. For more information about the options, see [Understanding Environment Maintenance](https://docs.cloud.oracle.com/iaas/Content/fusion-applications/plan-environment-family.htm#about-env-maintenance).
        """
        return pulumi.get(self, "maintenance_policy")

    @maintenance_policy.setter
    def maintenance_policy(self, value: Optional[pulumi.Input['FusionEnvironmentMaintenancePolicyArgs']]):
        pulumi.set(self, "maintenance_policy", value)

    @property
    @pulumi.getter
    def rules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRuleArgs']]]]:
        """
        (Updatable) Rules.
        """
        return pulumi.get(self, "rules")

    @rules.setter
    def rules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRuleArgs']]]]):
        pulumi.set(self, "rules", value)


@pulumi.input_type
class _FusionEnvironmentState:
    def __init__(__self__, *,
                 additional_language_packs: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 applied_patch_bundles: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 create_fusion_environment_admin_user_details: Optional[pulumi.Input['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 dns_prefix: Optional[pulumi.Input[str]] = None,
                 domain_id: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 fusion_environment_family_id: Optional[pulumi.Input[str]] = None,
                 fusion_environment_type: Optional[pulumi.Input[str]] = None,
                 idcs_domain_url: Optional[pulumi.Input[str]] = None,
                 is_break_glass_enabled: Optional[pulumi.Input[bool]] = None,
                 kms_key_id: Optional[pulumi.Input[str]] = None,
                 kms_key_infos: Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentKmsKeyInfoArgs']]]] = None,
                 lifecycle_details: Optional[pulumi.Input[str]] = None,
                 lockbox_id: Optional[pulumi.Input[str]] = None,
                 maintenance_policy: Optional[pulumi.Input['FusionEnvironmentMaintenancePolicyArgs']] = None,
                 public_url: Optional[pulumi.Input[str]] = None,
                 refreshes: Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRefreshArgs']]]] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRuleArgs']]]] = None,
                 state: Optional[pulumi.Input[str]] = None,
                 subscription_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 system_name: Optional[pulumi.Input[str]] = None,
                 time_created: Optional[pulumi.Input[str]] = None,
                 time_upcoming_maintenance: Optional[pulumi.Input[str]] = None,
                 time_updated: Optional[pulumi.Input[str]] = None,
                 version: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering FusionEnvironment resources.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] additional_language_packs: (Updatable) Language packs.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] applied_patch_bundles: Patch bundle names
        :param pulumi.Input[str] compartment_id: (Updatable) The unique identifier (OCID) of the compartment where the Fusion Environment is located.
        :param pulumi.Input['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs'] create_fusion_environment_admin_user_details: The credentials for the Fusion Applications service administrator.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        :param pulumi.Input[str] display_name: (Updatable) FusionEnvironment Identifier can be renamed.
        :param pulumi.Input[str] dns_prefix: DNS prefix.
        :param pulumi.Input[str] domain_id: The IDCS domain created for the fusion instance
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        :param pulumi.Input[str] fusion_environment_family_id: The unique identifier (OCID) of the Fusion Environment Family that the Fusion Environment belongs to.
        :param pulumi.Input[str] fusion_environment_type: The type of environment. Valid values are Production, Test, or Development.
        :param pulumi.Input[str] idcs_domain_url: The IDCS Domain URL
        :param pulumi.Input[bool] is_break_glass_enabled: If it's true, then the Break Glass feature is enabled
        :param pulumi.Input[str] kms_key_id: (Updatable) byok kms keyId
        :param pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentKmsKeyInfoArgs']]] kms_key_infos: BYOK key info
        :param pulumi.Input[str] lifecycle_details: A message describing the current state in more detail. For example, can be used to provide actionable information for a resource in Failed state.
        :param pulumi.Input[str] lockbox_id: The lockbox Id of this fusion environment. If there's no lockbox id, this field will be null
        :param pulumi.Input['FusionEnvironmentMaintenancePolicyArgs'] maintenance_policy: (Updatable) The policy that specifies the maintenance and upgrade preferences for an environment. For more information about the options, see [Understanding Environment Maintenance](https://docs.cloud.oracle.com/iaas/Content/fusion-applications/plan-environment-family.htm#about-env-maintenance).
        :param pulumi.Input[str] public_url: Public URL
        :param pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRefreshArgs']]] refreshes: Describes a refresh of a fusion environment
        :param pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRuleArgs']]] rules: (Updatable) Rules.
        :param pulumi.Input[str] state: The current state of the ServiceInstance.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] subscription_ids: List of subscription IDs.
        :param pulumi.Input[str] system_name: Environment Specific Guid/ System Name
        :param pulumi.Input[str] time_created: The time the the FusionEnvironment was created. An RFC3339 formatted datetime string
        :param pulumi.Input[str] time_upcoming_maintenance: The next maintenance for this environment
        :param pulumi.Input[str] time_updated: The time the FusionEnvironment was updated. An RFC3339 formatted datetime string
        :param pulumi.Input[str] version: Version of Fusion Apps used by this environment
        """
        if additional_language_packs is not None:
            pulumi.set(__self__, "additional_language_packs", additional_language_packs)
        if applied_patch_bundles is not None:
            pulumi.set(__self__, "applied_patch_bundles", applied_patch_bundles)
        if compartment_id is not None:
            pulumi.set(__self__, "compartment_id", compartment_id)
        if create_fusion_environment_admin_user_details is not None:
            pulumi.set(__self__, "create_fusion_environment_admin_user_details", create_fusion_environment_admin_user_details)
        if defined_tags is not None:
            pulumi.set(__self__, "defined_tags", defined_tags)
        if display_name is not None:
            pulumi.set(__self__, "display_name", display_name)
        if dns_prefix is not None:
            pulumi.set(__self__, "dns_prefix", dns_prefix)
        if domain_id is not None:
            pulumi.set(__self__, "domain_id", domain_id)
        if freeform_tags is not None:
            pulumi.set(__self__, "freeform_tags", freeform_tags)
        if fusion_environment_family_id is not None:
            pulumi.set(__self__, "fusion_environment_family_id", fusion_environment_family_id)
        if fusion_environment_type is not None:
            pulumi.set(__self__, "fusion_environment_type", fusion_environment_type)
        if idcs_domain_url is not None:
            pulumi.set(__self__, "idcs_domain_url", idcs_domain_url)
        if is_break_glass_enabled is not None:
            pulumi.set(__self__, "is_break_glass_enabled", is_break_glass_enabled)
        if kms_key_id is not None:
            pulumi.set(__self__, "kms_key_id", kms_key_id)
        if kms_key_infos is not None:
            pulumi.set(__self__, "kms_key_infos", kms_key_infos)
        if lifecycle_details is not None:
            pulumi.set(__self__, "lifecycle_details", lifecycle_details)
        if lockbox_id is not None:
            pulumi.set(__self__, "lockbox_id", lockbox_id)
        if maintenance_policy is not None:
            pulumi.set(__self__, "maintenance_policy", maintenance_policy)
        if public_url is not None:
            pulumi.set(__self__, "public_url", public_url)
        if refreshes is not None:
            pulumi.set(__self__, "refreshes", refreshes)
        if rules is not None:
            pulumi.set(__self__, "rules", rules)
        if state is not None:
            pulumi.set(__self__, "state", state)
        if subscription_ids is not None:
            pulumi.set(__self__, "subscription_ids", subscription_ids)
        if system_name is not None:
            pulumi.set(__self__, "system_name", system_name)
        if time_created is not None:
            pulumi.set(__self__, "time_created", time_created)
        if time_upcoming_maintenance is not None:
            pulumi.set(__self__, "time_upcoming_maintenance", time_upcoming_maintenance)
        if time_updated is not None:
            pulumi.set(__self__, "time_updated", time_updated)
        if version is not None:
            pulumi.set(__self__, "version", version)

    @property
    @pulumi.getter(name="additionalLanguagePacks")
    def additional_language_packs(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        (Updatable) Language packs.
        """
        return pulumi.get(self, "additional_language_packs")

    @additional_language_packs.setter
    def additional_language_packs(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "additional_language_packs", value)

    @property
    @pulumi.getter(name="appliedPatchBundles")
    def applied_patch_bundles(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Patch bundle names
        """
        return pulumi.get(self, "applied_patch_bundles")

    @applied_patch_bundles.setter
    def applied_patch_bundles(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "applied_patch_bundles", value)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The unique identifier (OCID) of the compartment where the Fusion Environment is located.
        """
        return pulumi.get(self, "compartment_id")

    @compartment_id.setter
    def compartment_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "compartment_id", value)

    @property
    @pulumi.getter(name="createFusionEnvironmentAdminUserDetails")
    def create_fusion_environment_admin_user_details(self) -> Optional[pulumi.Input['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']]:
        """
        The credentials for the Fusion Applications service administrator.
        """
        return pulumi.get(self, "create_fusion_environment_admin_user_details")

    @create_fusion_environment_admin_user_details.setter
    def create_fusion_environment_admin_user_details(self, value: Optional[pulumi.Input['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']]):
        pulumi.set(self, "create_fusion_environment_admin_user_details", value)

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        """
        return pulumi.get(self, "defined_tags")

    @defined_tags.setter
    def defined_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "defined_tags", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) FusionEnvironment Identifier can be renamed.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="dnsPrefix")
    def dns_prefix(self) -> Optional[pulumi.Input[str]]:
        """
        DNS prefix.
        """
        return pulumi.get(self, "dns_prefix")

    @dns_prefix.setter
    def dns_prefix(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dns_prefix", value)

    @property
    @pulumi.getter(name="domainId")
    def domain_id(self) -> Optional[pulumi.Input[str]]:
        """
        The IDCS domain created for the fusion instance
        """
        return pulumi.get(self, "domain_id")

    @domain_id.setter
    def domain_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "domain_id", value)

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        """
        return pulumi.get(self, "freeform_tags")

    @freeform_tags.setter
    def freeform_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "freeform_tags", value)

    @property
    @pulumi.getter(name="fusionEnvironmentFamilyId")
    def fusion_environment_family_id(self) -> Optional[pulumi.Input[str]]:
        """
        The unique identifier (OCID) of the Fusion Environment Family that the Fusion Environment belongs to.
        """
        return pulumi.get(self, "fusion_environment_family_id")

    @fusion_environment_family_id.setter
    def fusion_environment_family_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "fusion_environment_family_id", value)

    @property
    @pulumi.getter(name="fusionEnvironmentType")
    def fusion_environment_type(self) -> Optional[pulumi.Input[str]]:
        """
        The type of environment. Valid values are Production, Test, or Development.
        """
        return pulumi.get(self, "fusion_environment_type")

    @fusion_environment_type.setter
    def fusion_environment_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "fusion_environment_type", value)

    @property
    @pulumi.getter(name="idcsDomainUrl")
    def idcs_domain_url(self) -> Optional[pulumi.Input[str]]:
        """
        The IDCS Domain URL
        """
        return pulumi.get(self, "idcs_domain_url")

    @idcs_domain_url.setter
    def idcs_domain_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "idcs_domain_url", value)

    @property
    @pulumi.getter(name="isBreakGlassEnabled")
    def is_break_glass_enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        If it's true, then the Break Glass feature is enabled
        """
        return pulumi.get(self, "is_break_glass_enabled")

    @is_break_glass_enabled.setter
    def is_break_glass_enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "is_break_glass_enabled", value)

    @property
    @pulumi.getter(name="kmsKeyId")
    def kms_key_id(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) byok kms keyId
        """
        return pulumi.get(self, "kms_key_id")

    @kms_key_id.setter
    def kms_key_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kms_key_id", value)

    @property
    @pulumi.getter(name="kmsKeyInfos")
    def kms_key_infos(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentKmsKeyInfoArgs']]]]:
        """
        BYOK key info
        """
        return pulumi.get(self, "kms_key_infos")

    @kms_key_infos.setter
    def kms_key_infos(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentKmsKeyInfoArgs']]]]):
        pulumi.set(self, "kms_key_infos", value)

    @property
    @pulumi.getter(name="lifecycleDetails")
    def lifecycle_details(self) -> Optional[pulumi.Input[str]]:
        """
        A message describing the current state in more detail. For example, can be used to provide actionable information for a resource in Failed state.
        """
        return pulumi.get(self, "lifecycle_details")

    @lifecycle_details.setter
    def lifecycle_details(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "lifecycle_details", value)

    @property
    @pulumi.getter(name="lockboxId")
    def lockbox_id(self) -> Optional[pulumi.Input[str]]:
        """
        The lockbox Id of this fusion environment. If there's no lockbox id, this field will be null
        """
        return pulumi.get(self, "lockbox_id")

    @lockbox_id.setter
    def lockbox_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "lockbox_id", value)

    @property
    @pulumi.getter(name="maintenancePolicy")
    def maintenance_policy(self) -> Optional[pulumi.Input['FusionEnvironmentMaintenancePolicyArgs']]:
        """
        (Updatable) The policy that specifies the maintenance and upgrade preferences for an environment. For more information about the options, see [Understanding Environment Maintenance](https://docs.cloud.oracle.com/iaas/Content/fusion-applications/plan-environment-family.htm#about-env-maintenance).
        """
        return pulumi.get(self, "maintenance_policy")

    @maintenance_policy.setter
    def maintenance_policy(self, value: Optional[pulumi.Input['FusionEnvironmentMaintenancePolicyArgs']]):
        pulumi.set(self, "maintenance_policy", value)

    @property
    @pulumi.getter(name="publicUrl")
    def public_url(self) -> Optional[pulumi.Input[str]]:
        """
        Public URL
        """
        return pulumi.get(self, "public_url")

    @public_url.setter
    def public_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "public_url", value)

    @property
    @pulumi.getter
    def refreshes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRefreshArgs']]]]:
        """
        Describes a refresh of a fusion environment
        """
        return pulumi.get(self, "refreshes")

    @refreshes.setter
    def refreshes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRefreshArgs']]]]):
        pulumi.set(self, "refreshes", value)

    @property
    @pulumi.getter
    def rules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRuleArgs']]]]:
        """
        (Updatable) Rules.
        """
        return pulumi.get(self, "rules")

    @rules.setter
    def rules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FusionEnvironmentRuleArgs']]]]):
        pulumi.set(self, "rules", value)

    @property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[str]]:
        """
        The current state of the ServiceInstance.
        """
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "state", value)

    @property
    @pulumi.getter(name="subscriptionIds")
    def subscription_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        List of subscription IDs.
        """
        return pulumi.get(self, "subscription_ids")

    @subscription_ids.setter
    def subscription_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "subscription_ids", value)

    @property
    @pulumi.getter(name="systemName")
    def system_name(self) -> Optional[pulumi.Input[str]]:
        """
        Environment Specific Guid/ System Name
        """
        return pulumi.get(self, "system_name")

    @system_name.setter
    def system_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "system_name", value)

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> Optional[pulumi.Input[str]]:
        """
        The time the the FusionEnvironment was created. An RFC3339 formatted datetime string
        """
        return pulumi.get(self, "time_created")

    @time_created.setter
    def time_created(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_created", value)

    @property
    @pulumi.getter(name="timeUpcomingMaintenance")
    def time_upcoming_maintenance(self) -> Optional[pulumi.Input[str]]:
        """
        The next maintenance for this environment
        """
        return pulumi.get(self, "time_upcoming_maintenance")

    @time_upcoming_maintenance.setter
    def time_upcoming_maintenance(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_upcoming_maintenance", value)

    @property
    @pulumi.getter(name="timeUpdated")
    def time_updated(self) -> Optional[pulumi.Input[str]]:
        """
        The time the FusionEnvironment was updated. An RFC3339 formatted datetime string
        """
        return pulumi.get(self, "time_updated")

    @time_updated.setter
    def time_updated(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_updated", value)

    @property
    @pulumi.getter
    def version(self) -> Optional[pulumi.Input[str]]:
        """
        Version of Fusion Apps used by this environment
        """
        return pulumi.get(self, "version")

    @version.setter
    def version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "version", value)


class FusionEnvironment(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 additional_language_packs: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 create_fusion_environment_admin_user_details: Optional[pulumi.Input[pulumi.InputType['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 dns_prefix: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 fusion_environment_family_id: Optional[pulumi.Input[str]] = None,
                 fusion_environment_type: Optional[pulumi.Input[str]] = None,
                 kms_key_id: Optional[pulumi.Input[str]] = None,
                 maintenance_policy: Optional[pulumi.Input[pulumi.InputType['FusionEnvironmentMaintenancePolicyArgs']]] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FusionEnvironmentRuleArgs']]]]] = None,
                 __props__=None):
        """
        This resource provides the Fusion Environment resource in Oracle Cloud Infrastructure Fusion Apps service.

        Creates a new FusionEnvironment.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_fusion_environment = oci.fusion_apps.FusionEnvironment("testFusionEnvironment",
            compartment_id=var["compartment_id"],
            create_fusion_environment_admin_user_details=oci.fusion_apps.FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs(
                email_address=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                first_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                last_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                password=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                username=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            ),
            display_name=var["fusion_environment_display_name"],
            fusion_environment_family_id=oci_fusion_apps_fusion_environment_family["test_fusion_environment_family"]["id"],
            fusion_environment_type=var["fusion_environment_fusion_environment_type"],
            additional_language_packs=var["fusion_environment_additional_language_packs"],
            defined_tags={
                "foo-namespace.bar-key": "value",
            },
            dns_prefix=var["fusion_environment_dns_prefix"],
            freeform_tags={
                "bar-key": "value",
            },
            kms_key_id=oci_kms_key["test_key"]["id"],
            maintenance_policy=oci.fusion_apps.FusionEnvironmentMaintenancePolicyArgs(
                environment_maintenance_override=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                monthly_patching_override=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            ),
            rules=[oci.fusion_apps.FusionEnvironmentRuleArgs(
                action=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                conditions=[oci.fusion_apps.FusionEnvironmentRuleConditionArgs(
                    attribute_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    attribute_value=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                )],
                description=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            )])
        ```

        ## Import

        FusionEnvironments can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:FusionApps/fusionEnvironment:FusionEnvironment test_fusion_environment "id"
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] additional_language_packs: (Updatable) Language packs.
        :param pulumi.Input[str] compartment_id: (Updatable) The unique identifier (OCID) of the compartment where the Fusion Environment is located.
        :param pulumi.Input[pulumi.InputType['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']] create_fusion_environment_admin_user_details: The credentials for the Fusion Applications service administrator.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        :param pulumi.Input[str] display_name: (Updatable) FusionEnvironment Identifier can be renamed.
        :param pulumi.Input[str] dns_prefix: DNS prefix.
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        :param pulumi.Input[str] fusion_environment_family_id: The unique identifier (OCID) of the Fusion Environment Family that the Fusion Environment belongs to.
        :param pulumi.Input[str] fusion_environment_type: The type of environment. Valid values are Production, Test, or Development.
        :param pulumi.Input[str] kms_key_id: (Updatable) byok kms keyId
        :param pulumi.Input[pulumi.InputType['FusionEnvironmentMaintenancePolicyArgs']] maintenance_policy: (Updatable) The policy that specifies the maintenance and upgrade preferences for an environment. For more information about the options, see [Understanding Environment Maintenance](https://docs.cloud.oracle.com/iaas/Content/fusion-applications/plan-environment-family.htm#about-env-maintenance).
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FusionEnvironmentRuleArgs']]]] rules: (Updatable) Rules.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: FusionEnvironmentArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        This resource provides the Fusion Environment resource in Oracle Cloud Infrastructure Fusion Apps service.

        Creates a new FusionEnvironment.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_fusion_environment = oci.fusion_apps.FusionEnvironment("testFusionEnvironment",
            compartment_id=var["compartment_id"],
            create_fusion_environment_admin_user_details=oci.fusion_apps.FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs(
                email_address=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                first_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                last_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                password=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                username=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            ),
            display_name=var["fusion_environment_display_name"],
            fusion_environment_family_id=oci_fusion_apps_fusion_environment_family["test_fusion_environment_family"]["id"],
            fusion_environment_type=var["fusion_environment_fusion_environment_type"],
            additional_language_packs=var["fusion_environment_additional_language_packs"],
            defined_tags={
                "foo-namespace.bar-key": "value",
            },
            dns_prefix=var["fusion_environment_dns_prefix"],
            freeform_tags={
                "bar-key": "value",
            },
            kms_key_id=oci_kms_key["test_key"]["id"],
            maintenance_policy=oci.fusion_apps.FusionEnvironmentMaintenancePolicyArgs(
                environment_maintenance_override=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                monthly_patching_override=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            ),
            rules=[oci.fusion_apps.FusionEnvironmentRuleArgs(
                action=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                conditions=[oci.fusion_apps.FusionEnvironmentRuleConditionArgs(
                    attribute_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                    attribute_value=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                )],
                description=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            )])
        ```

        ## Import

        FusionEnvironments can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:FusionApps/fusionEnvironment:FusionEnvironment test_fusion_environment "id"
        ```

        :param str resource_name: The name of the resource.
        :param FusionEnvironmentArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(FusionEnvironmentArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 additional_language_packs: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 create_fusion_environment_admin_user_details: Optional[pulumi.Input[pulumi.InputType['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 dns_prefix: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 fusion_environment_family_id: Optional[pulumi.Input[str]] = None,
                 fusion_environment_type: Optional[pulumi.Input[str]] = None,
                 kms_key_id: Optional[pulumi.Input[str]] = None,
                 maintenance_policy: Optional[pulumi.Input[pulumi.InputType['FusionEnvironmentMaintenancePolicyArgs']]] = None,
                 rules: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FusionEnvironmentRuleArgs']]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = FusionEnvironmentArgs.__new__(FusionEnvironmentArgs)

            __props__.__dict__["additional_language_packs"] = additional_language_packs
            if compartment_id is None and not opts.urn:
                raise TypeError("Missing required property 'compartment_id'")
            __props__.__dict__["compartment_id"] = compartment_id
            if create_fusion_environment_admin_user_details is None and not opts.urn:
                raise TypeError("Missing required property 'create_fusion_environment_admin_user_details'")
            __props__.__dict__["create_fusion_environment_admin_user_details"] = create_fusion_environment_admin_user_details
            __props__.__dict__["defined_tags"] = defined_tags
            if display_name is None and not opts.urn:
                raise TypeError("Missing required property 'display_name'")
            __props__.__dict__["display_name"] = display_name
            __props__.__dict__["dns_prefix"] = dns_prefix
            __props__.__dict__["freeform_tags"] = freeform_tags
            if fusion_environment_family_id is None and not opts.urn:
                raise TypeError("Missing required property 'fusion_environment_family_id'")
            __props__.__dict__["fusion_environment_family_id"] = fusion_environment_family_id
            if fusion_environment_type is None and not opts.urn:
                raise TypeError("Missing required property 'fusion_environment_type'")
            __props__.__dict__["fusion_environment_type"] = fusion_environment_type
            __props__.__dict__["kms_key_id"] = kms_key_id
            __props__.__dict__["maintenance_policy"] = maintenance_policy
            __props__.__dict__["rules"] = rules
            __props__.__dict__["applied_patch_bundles"] = None
            __props__.__dict__["domain_id"] = None
            __props__.__dict__["idcs_domain_url"] = None
            __props__.__dict__["is_break_glass_enabled"] = None
            __props__.__dict__["kms_key_infos"] = None
            __props__.__dict__["lifecycle_details"] = None
            __props__.__dict__["lockbox_id"] = None
            __props__.__dict__["public_url"] = None
            __props__.__dict__["refreshes"] = None
            __props__.__dict__["state"] = None
            __props__.__dict__["subscription_ids"] = None
            __props__.__dict__["system_name"] = None
            __props__.__dict__["time_created"] = None
            __props__.__dict__["time_upcoming_maintenance"] = None
            __props__.__dict__["time_updated"] = None
            __props__.__dict__["version"] = None
        super(FusionEnvironment, __self__).__init__(
            'oci:FusionApps/fusionEnvironment:FusionEnvironment',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            additional_language_packs: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            applied_patch_bundles: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            compartment_id: Optional[pulumi.Input[str]] = None,
            create_fusion_environment_admin_user_details: Optional[pulumi.Input[pulumi.InputType['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']]] = None,
            defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
            display_name: Optional[pulumi.Input[str]] = None,
            dns_prefix: Optional[pulumi.Input[str]] = None,
            domain_id: Optional[pulumi.Input[str]] = None,
            freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
            fusion_environment_family_id: Optional[pulumi.Input[str]] = None,
            fusion_environment_type: Optional[pulumi.Input[str]] = None,
            idcs_domain_url: Optional[pulumi.Input[str]] = None,
            is_break_glass_enabled: Optional[pulumi.Input[bool]] = None,
            kms_key_id: Optional[pulumi.Input[str]] = None,
            kms_key_infos: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FusionEnvironmentKmsKeyInfoArgs']]]]] = None,
            lifecycle_details: Optional[pulumi.Input[str]] = None,
            lockbox_id: Optional[pulumi.Input[str]] = None,
            maintenance_policy: Optional[pulumi.Input[pulumi.InputType['FusionEnvironmentMaintenancePolicyArgs']]] = None,
            public_url: Optional[pulumi.Input[str]] = None,
            refreshes: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FusionEnvironmentRefreshArgs']]]]] = None,
            rules: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FusionEnvironmentRuleArgs']]]]] = None,
            state: Optional[pulumi.Input[str]] = None,
            subscription_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            system_name: Optional[pulumi.Input[str]] = None,
            time_created: Optional[pulumi.Input[str]] = None,
            time_upcoming_maintenance: Optional[pulumi.Input[str]] = None,
            time_updated: Optional[pulumi.Input[str]] = None,
            version: Optional[pulumi.Input[str]] = None) -> 'FusionEnvironment':
        """
        Get an existing FusionEnvironment resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] additional_language_packs: (Updatable) Language packs.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] applied_patch_bundles: Patch bundle names
        :param pulumi.Input[str] compartment_id: (Updatable) The unique identifier (OCID) of the compartment where the Fusion Environment is located.
        :param pulumi.Input[pulumi.InputType['FusionEnvironmentCreateFusionEnvironmentAdminUserDetailsArgs']] create_fusion_environment_admin_user_details: The credentials for the Fusion Applications service administrator.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        :param pulumi.Input[str] display_name: (Updatable) FusionEnvironment Identifier can be renamed.
        :param pulumi.Input[str] dns_prefix: DNS prefix.
        :param pulumi.Input[str] domain_id: The IDCS domain created for the fusion instance
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        :param pulumi.Input[str] fusion_environment_family_id: The unique identifier (OCID) of the Fusion Environment Family that the Fusion Environment belongs to.
        :param pulumi.Input[str] fusion_environment_type: The type of environment. Valid values are Production, Test, or Development.
        :param pulumi.Input[str] idcs_domain_url: The IDCS Domain URL
        :param pulumi.Input[bool] is_break_glass_enabled: If it's true, then the Break Glass feature is enabled
        :param pulumi.Input[str] kms_key_id: (Updatable) byok kms keyId
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FusionEnvironmentKmsKeyInfoArgs']]]] kms_key_infos: BYOK key info
        :param pulumi.Input[str] lifecycle_details: A message describing the current state in more detail. For example, can be used to provide actionable information for a resource in Failed state.
        :param pulumi.Input[str] lockbox_id: The lockbox Id of this fusion environment. If there's no lockbox id, this field will be null
        :param pulumi.Input[pulumi.InputType['FusionEnvironmentMaintenancePolicyArgs']] maintenance_policy: (Updatable) The policy that specifies the maintenance and upgrade preferences for an environment. For more information about the options, see [Understanding Environment Maintenance](https://docs.cloud.oracle.com/iaas/Content/fusion-applications/plan-environment-family.htm#about-env-maintenance).
        :param pulumi.Input[str] public_url: Public URL
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FusionEnvironmentRefreshArgs']]]] refreshes: Describes a refresh of a fusion environment
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FusionEnvironmentRuleArgs']]]] rules: (Updatable) Rules.
        :param pulumi.Input[str] state: The current state of the ServiceInstance.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] subscription_ids: List of subscription IDs.
        :param pulumi.Input[str] system_name: Environment Specific Guid/ System Name
        :param pulumi.Input[str] time_created: The time the the FusionEnvironment was created. An RFC3339 formatted datetime string
        :param pulumi.Input[str] time_upcoming_maintenance: The next maintenance for this environment
        :param pulumi.Input[str] time_updated: The time the FusionEnvironment was updated. An RFC3339 formatted datetime string
        :param pulumi.Input[str] version: Version of Fusion Apps used by this environment
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _FusionEnvironmentState.__new__(_FusionEnvironmentState)

        __props__.__dict__["additional_language_packs"] = additional_language_packs
        __props__.__dict__["applied_patch_bundles"] = applied_patch_bundles
        __props__.__dict__["compartment_id"] = compartment_id
        __props__.__dict__["create_fusion_environment_admin_user_details"] = create_fusion_environment_admin_user_details
        __props__.__dict__["defined_tags"] = defined_tags
        __props__.__dict__["display_name"] = display_name
        __props__.__dict__["dns_prefix"] = dns_prefix
        __props__.__dict__["domain_id"] = domain_id
        __props__.__dict__["freeform_tags"] = freeform_tags
        __props__.__dict__["fusion_environment_family_id"] = fusion_environment_family_id
        __props__.__dict__["fusion_environment_type"] = fusion_environment_type
        __props__.__dict__["idcs_domain_url"] = idcs_domain_url
        __props__.__dict__["is_break_glass_enabled"] = is_break_glass_enabled
        __props__.__dict__["kms_key_id"] = kms_key_id
        __props__.__dict__["kms_key_infos"] = kms_key_infos
        __props__.__dict__["lifecycle_details"] = lifecycle_details
        __props__.__dict__["lockbox_id"] = lockbox_id
        __props__.__dict__["maintenance_policy"] = maintenance_policy
        __props__.__dict__["public_url"] = public_url
        __props__.__dict__["refreshes"] = refreshes
        __props__.__dict__["rules"] = rules
        __props__.__dict__["state"] = state
        __props__.__dict__["subscription_ids"] = subscription_ids
        __props__.__dict__["system_name"] = system_name
        __props__.__dict__["time_created"] = time_created
        __props__.__dict__["time_upcoming_maintenance"] = time_upcoming_maintenance
        __props__.__dict__["time_updated"] = time_updated
        __props__.__dict__["version"] = version
        return FusionEnvironment(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="additionalLanguagePacks")
    def additional_language_packs(self) -> pulumi.Output[Sequence[str]]:
        """
        (Updatable) Language packs.
        """
        return pulumi.get(self, "additional_language_packs")

    @property
    @pulumi.getter(name="appliedPatchBundles")
    def applied_patch_bundles(self) -> pulumi.Output[Sequence[str]]:
        """
        Patch bundle names
        """
        return pulumi.get(self, "applied_patch_bundles")

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> pulumi.Output[str]:
        """
        (Updatable) The unique identifier (OCID) of the compartment where the Fusion Environment is located.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="createFusionEnvironmentAdminUserDetails")
    def create_fusion_environment_admin_user_details(self) -> pulumi.Output['outputs.FusionEnvironmentCreateFusionEnvironmentAdminUserDetails']:
        """
        The credentials for the Fusion Applications service administrator.
        """
        return pulumi.get(self, "create_fusion_environment_admin_user_details")

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> pulumi.Output[Mapping[str, Any]]:
        """
        (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        """
        return pulumi.get(self, "defined_tags")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Output[str]:
        """
        (Updatable) FusionEnvironment Identifier can be renamed.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="dnsPrefix")
    def dns_prefix(self) -> pulumi.Output[str]:
        """
        DNS prefix.
        """
        return pulumi.get(self, "dns_prefix")

    @property
    @pulumi.getter(name="domainId")
    def domain_id(self) -> pulumi.Output[str]:
        """
        The IDCS domain created for the fusion instance
        """
        return pulumi.get(self, "domain_id")

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> pulumi.Output[Mapping[str, Any]]:
        """
        (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        """
        return pulumi.get(self, "freeform_tags")

    @property
    @pulumi.getter(name="fusionEnvironmentFamilyId")
    def fusion_environment_family_id(self) -> pulumi.Output[str]:
        """
        The unique identifier (OCID) of the Fusion Environment Family that the Fusion Environment belongs to.
        """
        return pulumi.get(self, "fusion_environment_family_id")

    @property
    @pulumi.getter(name="fusionEnvironmentType")
    def fusion_environment_type(self) -> pulumi.Output[str]:
        """
        The type of environment. Valid values are Production, Test, or Development.
        """
        return pulumi.get(self, "fusion_environment_type")

    @property
    @pulumi.getter(name="idcsDomainUrl")
    def idcs_domain_url(self) -> pulumi.Output[str]:
        """
        The IDCS Domain URL
        """
        return pulumi.get(self, "idcs_domain_url")

    @property
    @pulumi.getter(name="isBreakGlassEnabled")
    def is_break_glass_enabled(self) -> pulumi.Output[bool]:
        """
        If it's true, then the Break Glass feature is enabled
        """
        return pulumi.get(self, "is_break_glass_enabled")

    @property
    @pulumi.getter(name="kmsKeyId")
    def kms_key_id(self) -> pulumi.Output[str]:
        """
        (Updatable) byok kms keyId
        """
        return pulumi.get(self, "kms_key_id")

    @property
    @pulumi.getter(name="kmsKeyInfos")
    def kms_key_infos(self) -> pulumi.Output[Sequence['outputs.FusionEnvironmentKmsKeyInfo']]:
        """
        BYOK key info
        """
        return pulumi.get(self, "kms_key_infos")

    @property
    @pulumi.getter(name="lifecycleDetails")
    def lifecycle_details(self) -> pulumi.Output[str]:
        """
        A message describing the current state in more detail. For example, can be used to provide actionable information for a resource in Failed state.
        """
        return pulumi.get(self, "lifecycle_details")

    @property
    @pulumi.getter(name="lockboxId")
    def lockbox_id(self) -> pulumi.Output[str]:
        """
        The lockbox Id of this fusion environment. If there's no lockbox id, this field will be null
        """
        return pulumi.get(self, "lockbox_id")

    @property
    @pulumi.getter(name="maintenancePolicy")
    def maintenance_policy(self) -> pulumi.Output['outputs.FusionEnvironmentMaintenancePolicy']:
        """
        (Updatable) The policy that specifies the maintenance and upgrade preferences for an environment. For more information about the options, see [Understanding Environment Maintenance](https://docs.cloud.oracle.com/iaas/Content/fusion-applications/plan-environment-family.htm#about-env-maintenance).
        """
        return pulumi.get(self, "maintenance_policy")

    @property
    @pulumi.getter(name="publicUrl")
    def public_url(self) -> pulumi.Output[str]:
        """
        Public URL
        """
        return pulumi.get(self, "public_url")

    @property
    @pulumi.getter
    def refreshes(self) -> pulumi.Output[Sequence['outputs.FusionEnvironmentRefresh']]:
        """
        Describes a refresh of a fusion environment
        """
        return pulumi.get(self, "refreshes")

    @property
    @pulumi.getter
    def rules(self) -> pulumi.Output[Sequence['outputs.FusionEnvironmentRule']]:
        """
        (Updatable) Rules.
        """
        return pulumi.get(self, "rules")

    @property
    @pulumi.getter
    def state(self) -> pulumi.Output[str]:
        """
        The current state of the ServiceInstance.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="subscriptionIds")
    def subscription_ids(self) -> pulumi.Output[Sequence[str]]:
        """
        List of subscription IDs.
        """
        return pulumi.get(self, "subscription_ids")

    @property
    @pulumi.getter(name="systemName")
    def system_name(self) -> pulumi.Output[str]:
        """
        Environment Specific Guid/ System Name
        """
        return pulumi.get(self, "system_name")

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> pulumi.Output[str]:
        """
        The time the the FusionEnvironment was created. An RFC3339 formatted datetime string
        """
        return pulumi.get(self, "time_created")

    @property
    @pulumi.getter(name="timeUpcomingMaintenance")
    def time_upcoming_maintenance(self) -> pulumi.Output[str]:
        """
        The next maintenance for this environment
        """
        return pulumi.get(self, "time_upcoming_maintenance")

    @property
    @pulumi.getter(name="timeUpdated")
    def time_updated(self) -> pulumi.Output[str]:
        """
        The time the FusionEnvironment was updated. An RFC3339 formatted datetime string
        """
        return pulumi.get(self, "time_updated")

    @property
    @pulumi.getter
    def version(self) -> pulumi.Output[str]:
        """
        Version of Fusion Apps used by this environment
        """
        return pulumi.get(self, "version")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetTargetDatabasesTablesResult',
    'AwaitableGetTargetDatabasesTablesResult',
    'get_target_databases_tables',
    'get_target_databases_tables_output',
]

@pulumi.output_type
class GetTargetDatabasesTablesResult:
    """
    A collection of values returned by getTargetDatabasesTables.
    """
    def __init__(__self__, filters=None, id=None, schema_name_contains=None, schema_names=None, table_name_contains=None, table_names=None, tables=None, target_database_id=None):
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if schema_name_contains and not isinstance(schema_name_contains, str):
            raise TypeError("Expected argument 'schema_name_contains' to be a str")
        pulumi.set(__self__, "schema_name_contains", schema_name_contains)
        if schema_names and not isinstance(schema_names, list):
            raise TypeError("Expected argument 'schema_names' to be a list")
        pulumi.set(__self__, "schema_names", schema_names)
        if table_name_contains and not isinstance(table_name_contains, str):
            raise TypeError("Expected argument 'table_name_contains' to be a str")
        pulumi.set(__self__, "table_name_contains", table_name_contains)
        if table_names and not isinstance(table_names, list):
            raise TypeError("Expected argument 'table_names' to be a list")
        pulumi.set(__self__, "table_names", table_names)
        if tables and not isinstance(tables, list):
            raise TypeError("Expected argument 'tables' to be a list")
        pulumi.set(__self__, "tables", tables)
        if target_database_id and not isinstance(target_database_id, str):
            raise TypeError("Expected argument 'target_database_id' to be a str")
        pulumi.set(__self__, "target_database_id", target_database_id)

    @property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetTargetDatabasesTablesFilterResult']]:
        return pulumi.get(self, "filters")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="schemaNameContains")
    def schema_name_contains(self) -> Optional[str]:
        return pulumi.get(self, "schema_name_contains")

    @property
    @pulumi.getter(name="schemaNames")
    def schema_names(self) -> Optional[Sequence[str]]:
        """
        Name of the schema.
        """
        return pulumi.get(self, "schema_names")

    @property
    @pulumi.getter(name="tableNameContains")
    def table_name_contains(self) -> Optional[str]:
        return pulumi.get(self, "table_name_contains")

    @property
    @pulumi.getter(name="tableNames")
    def table_names(self) -> Optional[Sequence[str]]:
        """
        Name of the table.
        """
        return pulumi.get(self, "table_names")

    @property
    @pulumi.getter
    def tables(self) -> Sequence['outputs.GetTargetDatabasesTablesTableResult']:
        """
        The list of tables.
        """
        return pulumi.get(self, "tables")

    @property
    @pulumi.getter(name="targetDatabaseId")
    def target_database_id(self) -> str:
        return pulumi.get(self, "target_database_id")


class AwaitableGetTargetDatabasesTablesResult(GetTargetDatabasesTablesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetTargetDatabasesTablesResult(
            filters=self.filters,
            id=self.id,
            schema_name_contains=self.schema_name_contains,
            schema_names=self.schema_names,
            table_name_contains=self.table_name_contains,
            table_names=self.table_names,
            tables=self.tables,
            target_database_id=self.target_database_id)


def get_target_databases_tables(filters: Optional[Sequence[pulumi.InputType['GetTargetDatabasesTablesFilterArgs']]] = None,
                                schema_name_contains: Optional[str] = None,
                                schema_names: Optional[Sequence[str]] = None,
                                table_name_contains: Optional[str] = None,
                                table_names: Optional[Sequence[str]] = None,
                                target_database_id: Optional[str] = None,
                                opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetTargetDatabasesTablesResult:
    """
    This data source provides the list of Target Databases Tables in Oracle Cloud Infrastructure Data Safe service.

    Returns a list of table metadata objects.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_target_databases_tables = oci.DataSafe.get_target_databases_tables(target_database_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        schema_names=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        schema_name_contains=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        table_names=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        table_name_contains=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str schema_name_contains: A filter to return only items if schema name contains a specific string.
    :param Sequence[str] schema_names: A filter to return only items related to specific schema name.
    :param str table_name_contains: A filter to return only items if table name contains a specific string.
    :param Sequence[str] table_names: A filter to return only items related to specific table name.
    :param str target_database_id: The OCID of the Data Safe target database.
    """
    __args__ = dict()
    __args__['filters'] = filters
    __args__['schemaNameContains'] = schema_name_contains
    __args__['schemaNames'] = schema_names
    __args__['tableNameContains'] = table_name_contains
    __args__['tableNames'] = table_names
    __args__['targetDatabaseId'] = target_database_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:DataSafe/getTargetDatabasesTables:getTargetDatabasesTables', __args__, opts=opts, typ=GetTargetDatabasesTablesResult).value

    return AwaitableGetTargetDatabasesTablesResult(
        filters=__ret__.filters,
        id=__ret__.id,
        schema_name_contains=__ret__.schema_name_contains,
        schema_names=__ret__.schema_names,
        table_name_contains=__ret__.table_name_contains,
        table_names=__ret__.table_names,
        tables=__ret__.tables,
        target_database_id=__ret__.target_database_id)


@_utilities.lift_output_func(get_target_databases_tables)
def get_target_databases_tables_output(filters: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetTargetDatabasesTablesFilterArgs']]]]] = None,
                                       schema_name_contains: Optional[pulumi.Input[Optional[str]]] = None,
                                       schema_names: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                                       table_name_contains: Optional[pulumi.Input[Optional[str]]] = None,
                                       table_names: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                                       target_database_id: Optional[pulumi.Input[str]] = None,
                                       opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetTargetDatabasesTablesResult]:
    """
    This data source provides the list of Target Databases Tables in Oracle Cloud Infrastructure Data Safe service.

    Returns a list of table metadata objects.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_target_databases_tables = oci.DataSafe.get_target_databases_tables(target_database_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        schema_names=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        schema_name_contains=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        table_names=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        table_name_contains=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str schema_name_contains: A filter to return only items if schema name contains a specific string.
    :param Sequence[str] schema_names: A filter to return only items related to specific schema name.
    :param str table_name_contains: A filter to return only items if table name contains a specific string.
    :param Sequence[str] table_names: A filter to return only items related to specific table name.
    :param str target_database_id: The OCID of the Data Safe target database.
    """
    ...
